@@ -3,6 +3,7 @@
 import numpy as np
 from scipy.optimize import minimize
 from functools import partial
+import matplotlib.pyplot as plt
 
 
 from sklearn.cluster import KMeans
@@ -206,68 +207,154 @@ class nKmeansInit:
 
 
 
+#################################################################################################################################    
+
+
+def preprocess_data(xraw):
+    
+    x_sum = np.sum(xraw, axis=1)
+    xcopy = np.copy(xraw)
+    check_pix = False
+    check_level = False
+    deletedpix = 0
+    deletedLevels = 0
     
     
     
-
-
-
-
-
-def pcanipals(Xh,A,it=1000,tol=1e-4):
-   
     
-    obsCount,varCount = np.shape(Xh);
-    T = np.zeros([obsCount,A]);
-    P = np.zeros([varCount,A]);
-    nr = 0;
+    #we take out every pixels which did not receive any count. We will add them back at the end 
     
-    for a in range(A):
+    if np.any(x_sum == 0):
         
-        th = np.array([Xh[:,0]]).T;
-        ende = False;
-    
-        while(ende==False):
+        print('One or more pixel(s) did not receive any count so they were taken out for the calculation')
             
-            nr+=1
-            ph=Xh.T@th/np.linalg.inv(th.T@th)
-            ph=ph/np.linalg.norm(ph)
-            thnew=Xh@ph
-            prec = (thnew-th).T@(thnew-th)
-            th=thnew
+        check_pix = True
+        deletedpix = np.where(x_sum == 0)[0]
+        xcopy = np.delete(xraw, deletedpix, axis = 0)
+        x_sum = np.delete(x_sum , deletedpix)
+        
+    
+    
+    
+    #Avoid errors, if an energy level has 0 counts on all pixels, it causes log(0) and the level is meaningless, so we take it out
+    
+    sum_level = np.sum(xcopy , axis = 0)
+    
+    if np.any(sum_level == 0):
+        
+        print('One or more level of energy did not have any count on it so they were taken out for calculation')
+        check_level = True
+        
+        deletedLevels = np.where(sum_level == 0)[0]
+        
+        xcopy = np.delete(xcopy , deletedLevels , axis = 1 )
+        
+
+
+    
+    #We finally normalize each spectra so their sum is zero
+        
+    x_sum = np.array([np.sum(xcopy, axis=1)]).T
+    x = xcopy / x_sum
+    
+  
+    return x , x_sum , deletedpix , deletedLevels , check_pix , check_level
+    
+
+
+
+
+
+
+def final_process(C, S , deletedpix , deletedLevels, check_pix, check_level):
+    
+    
+    nb_c = np.shape(C)[1]
+    
+    # We add back the pixels we took out initially. The concentrations are all set to 0
+
+    if check_pix == True:
+        
+        nb_pix_ori = np.shape(C)[0] + len(deletedpix)
+        
+        
+        C_final = np.zeros([nb_pix_ori, nb_c])
+        
+        counter = 0
             
-            if(prec<=tol**2):
-                ende=True
-            elif(it<=nr):
-                ende=True
+        
+        for i in range(nb_pix_ori):
+            
+            if np.any(deletedpix == i):
+                
+                C_final[i,:] = 0
+                counter += 1
+            
+            else: 
+                C_final[i,:] = C[i-counter,:]
+                
     
-        Xh = Xh - th@ph.T
-        T[:,a] = th[:,0]
-        P[:,a] = ph[:,0]
-    
-    return T,P
+    else:
+        C_final = C
     
     
+    
+    
+    #Insert deleted levels back 
+    
+    if check_level == True:
+        
+        nb_level_ori = np.shape(S)[1] + len(deletedLevels)
+        
+        S_final = np.zeros([nb_c , nb_level_ori] )
 
+        counter = 0
+            
+        
+        for i in range(nb_level_ori):
+            
+            if np.any(deletedLevels == i):
+                
+                S_final[:,i] = (S[:,i-counter-1] + S[:,i-counter+1])/2
+                counter += 1
+            
+            else: 
+                S_final[:,i] = S[:,i-counter]
+                
+        
+    else:
+        S_final = S
+                
+        
+        
+        
+    return C_final,S_final
 
-
+    
+##############################################################################################################################################    
 
 
 class mcrllm:
     
     
-    def __init__(self,X,nb_c,method="standard",fact_ini=.5):
+    def __init__(self,Xraw,nb_c,init,nb_iter=1,method="variable",fact_ini=.5):
         
         # Save Xraw and normalize data
-        self.Xraw = X.copy()
-        self.Xsum = np.sum(X,axis=1)
-        self.X = X/np.array([np.sum(X,axis=1)]).T
+        self.Xraw = Xraw
+        self.X ,self.Xsum,self.deletedpix, self.deletedLevels, self.check_pix , self.check_level = preprocess_data(self.Xraw)
+        
+        if self.check_pix:
+            self.Xraw = np.delete(self.Xraw , self.deletedpix , axis = 0)
+        
+        if self.check_level:
+            self.Xraw = np.delete(self.Xraw , self.deletedLevels , axis = 1)
+        
         
         self.pix,self.var = np.shape(self.X)
         self.nb_c = nb_c
-        self.method = "phi" ################## self.method = method
+        self.method = method 
         self.expvar = np.inf
-        self.fact_ini = .5
+        self.fact_ini = fact_ini
         
         # History initialization
         self.allC = []
@@ -277,30 +364,25 @@ class mcrllm:
         
         self.C = np.ones([self.pix,self.nb_c])/nb_c
         
-        # PCA reconstruction
-        X_m = np.array([np.mean(self.X,axis=0)])
-        X_s = np.array([np.std(self.X,axis=0)])
-        Xpca = (self.X - X_m)/X_s
-        T,P = pcanipals(Xpca,nb_c-1)
-        self.Xrec = (T@P.T)*X_s+X_m
+
             
         # Initialization
-        self.Sini = KmeansInit.initialisation(self.X, nb_c)
-        #if(np.sum(self.Sini <= 0)>0):
-        #    raise("Bas initialization")
-        self.S = self.Sini.copy()
+        self.define_initial_spectra(init)
         
         
-    
-    def iterate(self,nb_iter=1):
         
         for iteration in range(nb_iter):
-            #print("Iteration {:.0f}".format(len(self.allS)+1))
+            print("Iteration {:.0f}".format(len(self.allS)+1))
+
+            
             self.C_plm()
             self.S_plm()
             
             
-            
+        self.C,self.S = final_process(self.C ,self.S, self.deletedpix , self.deletedLevels , self.check_pix , self.check_level )
+        
+        
+     
     
     def C_plm(self):
         
@@ -346,7 +428,7 @@ class mcrllm:
         h = np.random.permutation(len(self.X))
         phi_optimal = 1
         
-        if self.method == "phi":
+        if self.method == "variable":
             allMSE = []
             all_phis = np.arange(.1,10.1,.1)
             
@@ -424,6 +506,57 @@ class mcrllm:
         
         
         return c_new
+    
+    
+    
+    
+    
+    def define_initial_spectra(self,init):
+        
+        if type(init) == type(''):
+            
+            if init == 'Kmeans':
+                print('Initializing with {}'.format(init))
+                self.Sini = KmeansInit.initialisation(self.X,self.nb_c)
+                self.S = self.Sini.copy()
+                
+            elif init == 'MBKmeans':
+                print('Initializing with {}'.format(init))
+                self.Sini = MBKmeansInit.initialisation(self.X,self.nb_c)
+                self.S = self.Sini.copy()
+            
+            
+            elif init == 'NFindr':
+                print('Initializing with {}'.format(init))
+                self.Sini = NFindrInit.initialisation(self.X,self.nb_c)
+                self.S = self.Sini.copy()
+            
+            elif init == 'RobustNFindr':
+                print('Initializing with {}'.format(init))
+                self.Sini = RobustNFindrInit.initialisation(self.X,self.nb_c)
+                self.S = self.Sini.copy()
+                
+            elif init == 'ATGP':
+                print('Initializing with {}'.format(init))
+                self.Sini = AtgpInit.initialisation(self.X,self.nb_c)
+                self.S = self.Sini.copy()
+               
+            elif init == 'FIPPI':
+                print('Initializing with {}'.format(init))
+                self.Sini = FippiInit.initialisation(self.X,self.nb_c)
+                self.S = self.Sini.copy()
+                
+            elif init == 'nKmeans':
+                print('Initializing with {}'.format(init))
+                self.Sini = nKmeansInit.initialisation(self.X,self.nb_c)
+                self.S = self.Sini.copy()
+        
+        elif type(init) == type(np.array([1])):
+            print('Initializing with given spectra')
+            self.S = init
+            
+        else:
+            raise('Initialization method not found')
         
         
     
